#version 330 core

/* Source: http://www.opengl-tutorial.org/beginners-tutorials/tutorial-2-the-first-triangle/
"layout(location = 0)" refers to the buffer we use to feed the in_vertPos_modelSpace attribute. Each vertex can have numerous attributes : A position, one or several colours, one or several texture coordinates, lots of other things. OpenGL doesn’t know what a colour is : it just sees a vec3. So we have to tell him which buffer corresponds to which input. We do that by setting the layout to the same value as the first parameter to glVertexAttribPointer. The value “0” is not important, it could be 12 (but no more than glGetIntegerv(GL_MAX_VERTEX_ATTRIBS, &v) ), the important thing is that it’s the same number on both sides.
*/

layout(location = 0) in vec3 in_vertPos_modelSpace;
//layout(location = 1) in vec2 in_texCoord;
layout(location = 1) in vec3 in_normal;
layout(location = 2) in vec2 in_texCoords;
layout(location = 3) in vec3 in_tangent;
layout(location = 4) in vec3 in_bitangent;

out vec3 vert_fragColor;
out vec2 vert_texCoords;
out vec3 vert_normal;
out vec3 vert_fragPos;
out mat3 vert_tbn;
out vec4 viewSpace;

uniform mat4 u_modelMatrix;
uniform mat4 u_viewMatrix;
uniform mat4 u_viewProjMatrix;
uniform mat4 u_modelMatrixInverseTranspose;

void main() {
	viewSpace = u_viewMatrix * u_modelMatrix * vec4(in_vertPos_modelSpace, 1);
	gl_Position = (u_viewProjMatrix * u_modelMatrix) * vec4(in_vertPos_modelSpace, 1);
	//gl_Position = vec4(in_vertPos_modelSpace.x, in_vertPos_modelSpace.y, in_vertPos_modelSpace.z, 1.0);
	
	vert_texCoords = in_texCoords;
	vert_normal = vec3(u_modelMatrixInverseTranspose * vec4(in_normal, 1.0f));
	
	vec3 tangent   = normalize(vec3(u_modelMatrix * vec4(in_tangent,   0.0f)));
	vec3 bitangent = normalize(vec3(u_modelMatrix * vec4(in_bitangent, 0.0f)));
	vec3 normal    = normalize(vec3(u_modelMatrix * vec4(in_normal,    0.0f)));
	vert_tbn = mat3(tangent, bitangent, normal);
	
	vert_fragPos = (u_modelMatrix * vec4(in_vertPos_modelSpace, 1.0f)).rgb;
}