#version 330 core

#extension GL_NV_shadow_samplers_cube : enable

in vec2 vert_texCoords;
in vec3 vert_normal;
in vec3 vert_fragPos;

in mat3 vert_tbn;
in vec4 vert_fragPosViewSpace;

layout (location = 0) out vec4 out_fragColorSpec;
layout (location = 1) out vec4 out_fragPos;
layout (location = 2) out vec3 out_normal;
layout (location = 3) out vec4 out_viewSpace;

uniform vec3 u_camPos;

// Texturing
uniform vec3 u_color = vec3(1.0f, 0.0f, 1.0f); // magenta

uniform sampler2D u_texDiffuse; // 0
uniform bool u_useDiff;

uniform sampler2D u_texSpecular; // 1
uniform bool u_useSpec;

uniform sampler2D u_texNormal; // 2
uniform bool u_useNormal;

void main() {

	out_fragPos = vec4(vert_fragPos, length(u_camPos - vert_fragPos));
	
	if (u_useNormal) {
		out_normal = texture2D(u_texNormal, vert_texCoords).xyz;
		out_normal = normalize(out_normal * 2.0f - 1.0f);
		out_normal = normalize(vert_tbn * out_normal);
	} else {
		out_normal = normalize(vert_normal);
	}
	
	out_fragColorSpec.rgb = u_useDiff ? (texture2D(u_texDiffuse, vert_texCoords).rgb) : u_color;
	out_fragColorSpec.a = u_useSpec ? (texture2D(u_texSpecular, vert_texCoords).r) : 1.0f;
	
	out_viewSpace = vert_fragPosViewSpace;
}