#version 330 core

layout(location = 0) in vec3 in_vertPos_modelSpace;
layout(location = 1) in vec3 in_normal;
layout(location = 2) in vec2 in_texCoords;
layout(location = 3) in vec3 in_tangent;
layout(location = 4) in vec3 in_bitangent;

out vec2 vert_texCoords;
out vec3 vert_normal;
out vec3 vert_fragPos;
out mat3 vert_tbn;
out vec4 vert_fragPosViewSpace;

uniform mat4 u_modelMatrix;
uniform mat4 u_viewMatrix;
uniform mat4 u_viewProjMatrix;
uniform mat4 u_modelMatrixInverseTranspose;

void main() {
	vec4 fragPos = u_modelMatrix * vec4(in_vertPos_modelSpace, 1.0f);
	vert_fragPos = fragPos.xyz;
	vert_fragPosViewSpace = u_viewMatrix * fragPos;
	
	gl_Position = u_viewProjMatrix * vec4(vert_fragPos, 1.0f);
	
	vert_texCoords = in_texCoords;
	vert_normal = vec3(u_modelMatrixInverseTranspose * vec4(in_normal, 1.0f));
	
	vec3 tangent   = normalize(vec3(u_modelMatrix * vec4(in_tangent,   0.0f)));
	vec3 bitangent = normalize(vec3(u_modelMatrix * vec4(in_bitangent, 0.0f)));
	vec3 normal    = normalize(vec3(u_modelMatrix * vec4(in_normal,    0.0f)));
	vert_tbn = mat3(tangent, bitangent, normal);
}