#version 330 core

layout (points) in;
layout (triangle_strip, max_vertices = 4) out; // quads

uniform mat4 projMatrix;

in data {
	float vert_timeToLive;
	float vert_angle;
	int vert_type;
} vertex[];

out vec2 texCoord;
flat out float timeToLive;
flat out int type;
out vec3 fragPos;

uniform float baseSize = 1.0f;
uniform float inverseTimeToLiveTotal;

mat2 rot(float angle) {
	return mat2(cos(angle), -sin(angle),
				sin(angle),  cos(angle));
}

void main() {

	timeToLive = vertex[0].vert_timeToLive;
	
	if (timeToLive >= 0) {
	
		// This doesn't work :(
		/*if (type == 2.0f) { // thingy
			type = vertex[0].vert_type;
			float angle = vertex[0].vert_angle;
			mat2 rot = rot(angle);
			vec4 pos = gl_in[0].gl_Position; // in view space
			
			vec2 v;
			
			texCoord = vec2(0.0f, 0.0f);
			
			v = pos.xy + rot * vec2(-0.1f, 0.0f);
			gl_Position = projMatrix * vec4(v, pos.zw);
			EmitVertex();
			
			v = pos.xy + rot * vec2(0.1f, 0.0f);
			gl_Position = projMatrix * vec4(v, pos.zw);
			EmitVertex();
			
			v = pos.xy + rot * vec2(0.0f, 1.0f);
			gl_Position = projMatrix * vec4(v, pos.zw);
			EmitVertex();
			
			EndPrimitive();
			return;
		}*/
	
		float t = (1.0f - (timeToLive * inverseTimeToLiveTotal));
		t = clamp(t, 0.25f, 1.0f);
		
		type = vertex[0].vert_type;
		
		vec2 size = vec2(1.0f, 1.0f) * baseSize * t;
		vec4 pos = gl_in[0].gl_Position; // in view space
	
		float angle = vertex[0].vert_angle;
		mat2 rot = rot(angle);
	
		vec2 v;
		
		v = pos.xy + rot * (vec2(-0.5f, -0.5f) * size); // bottom-left vertex
		gl_Position = projMatrix * vec4(v, pos.zw);
		fragPos = gl_Position.xyz;
		texCoord = vec2(0.0f, 0.0f);
		EmitVertex();
		
		v = pos.xy + rot * (vec2(0.5f, -0.5f) * size); // bottom-right vertex
		gl_Position = projMatrix * vec4(v, pos.zw);
		fragPos = gl_Position.xyz;
		texCoord = vec2(1.0f, 0.0f);
		EmitVertex();
		
		v = pos.xy + rot * (vec2(-0.5f, 0.5f) * size); // top-right vertex
		gl_Position = projMatrix * vec4(v, pos.zw);
		fragPos = gl_Position.xyz;
		texCoord = vec2(0.0f, 1.0f);
		EmitVertex();
		
		v = pos.xy + rot * (vec2(0.5f, 0.5f) * size); // top-left vertex
		gl_Position = projMatrix * vec4(v, pos.zw);
		fragPos = gl_Position.xyz;
		texCoord = vec2(1.0f, 1.0f);
		EmitVertex();
		
		EndPrimitive();
	}
}