#version 330 core

in vec2 vert_texCoords;

out vec4 out_fragColor;

uniform sampler2D u_frontFaces;
uniform sampler2D u_backFaces;
uniform sampler3D u_volumePresssureGradient;
uniform sampler2D u_scenePosDis;

const float rayStepInc = 0.01f;

// Lighting
struct PointLight {
	vec3 pos;
	vec3 col;
	float spec;
	// attenuation
	float cons;
	float lin;
	float quad;
};
uniform PointLight u_pointLights[10];
uniform int u_pointLightsCount = 0;
uniform float u_lightAmb;  // ambient lighting intensity

vec3 illuminate(vec3 pos, vec3 normal) {
	vec3 color = vec3(0.0f, 0.0f, 0.0f);
	for (int i = 0; i < u_pointLightsCount; i++) {
		PointLight light = u_pointLights[i];
		vec3 posToLight = light.pos - pos;
		vec3 lightDir = normalize(posToLight);
		float dis = length(posToLight);
		vec3 diffuse = max(dot(normal, lightDir), 0.0f) * light.col;
		float attenuation = 1.0f / (light.cons + light.lin * dis + light.quad * (dis*dis));
		color += (attenuation * diffuse);
	}
	return color;
}

vec4 getColor(float pressure) {
	vec4 col1 = vec4(0.3f, 0.4f, 0.1f, 0.0001f); // low pressure
	vec4 col2 = vec4(0.58f, 0.933f, 0.055f, 1.0f); // high pressure
	return mix(col1, col2, pressure);
}

void main() {

	vec4 start = texture2D(u_frontFaces, vert_texCoords);
	vec3 startPos = start.xyz;
	float startDis = start.w;
	
	vec4 end = texture2D(u_backFaces, vert_texCoords);
	vec3 endPos = end.xyz;
	
	vec4 scene = texture2D(u_scenePosDis, vert_texCoords);
	float sceneDis = scene.w == 0 ? 10000.0f : scene.w;
	
	if (startPos == endPos || sceneDis < startDis) {
		discard;
	}
	
	vec3 ray = endPos - startPos;
	//float rayLen = length(ray);
	vec3 posStep = rayStepInc * normalize(ray);
	int stepCount = int(floor(length(ray) / rayStepInc));
	
	float endDis = end.w;
	float disTotal = endDis - startDis;
	float disStep = disTotal / stepCount;
	if (sceneDis < endDis) {
		disTotal = sceneDis - startDis;
	}
	
	vec3 pos = startPos;
	vec4 color = vec4(0.0f, 0.0f, 0.0f, 0.0f);
	for (float dis = 0.0f; dis < disTotal && color.a < 0.99f; dis += disStep, pos += posStep) {
	
		vec4 denGrad = texture(u_volumePresssureGradient, pos);
		float pressure = clamp(denGrad.r, 0.0f, 1.0f);
		vec3 gradient = denGrad.gba;
		
		if (pressure > 0) {
			vec4 colorCur = getColor(pressure);
			color.rgb = mix(color.rgb, colorCur.rgb, colorCur.a);
			color.a = mix(color.a, 1.0f, colorCur.a);
			
			//color.rgb += illuminate(pos, gradient);
		}
	}
	
	//vec3 pos = startPos;
	//float len = 0;
	//vec4 color = vec4(0.0f, 0.0f, 0.0f, 0.0f);
	//do {
	//	vec4 denGrad = texture(u_volumePresssureGradient, pos);
	//	float pressure = clamp(denGrad.r, 0.0f, 1.0f);
	//	vec3 gradient = denGrad.gba;
	//	
	//	if (pressure > 0) {
	//		vec4 colorCur = getColor(pressure);
	//		color.rgb = mix(color.rgb, colorCur.rgb, colorCur.a);
	//		color.a = mix(color.a, 1.0f, colorCur.a);
	//		
	//		//color.rgb += illuminate(pos, gradient);
	//	}
	//	
	//	pos += rayStep;
	//	len += rayStepInc;
	//} while (len <= rayLen && color.a < 0.99f);
	
	out_fragColor = color;
}